"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.h = exports.Fragment = exports.RawHtml = void 0;
class RawHtml {
    html;
    constructor(html) {
        this.html = html;
    }
    toString() {
        return this.html;
    }
}
exports.RawHtml = RawHtml;
function toRawHtml(element) {
    if (element === undefined || element === null) {
        return new RawHtml("");
    }
    else if (element instanceof RawHtml) {
        return element;
    }
    else if (Array.isArray(element)) {
        return new RawHtml(element.map((e) => toRawHtml(e).html).join(""));
    }
    else {
        return new RawHtml(escapeHtml(String(element)));
    }
}
function escapeHtml(text) {
    return text
        .replace(/&/g, "&amp;")
        .replace(/</g, "&lt;")
        .replace(/>/g, "&gt;");
}
function escapeAttribute(text) {
    return escapeHtml(text).replace(/'/g, "&#39;").replace(/"/g, "&quot;");
}
function toAttributes(props) {
    return Object.entries(props)
        .map(([name, value]) => ` ${name}="${escapeAttribute(String(value))}"`)
        .join("");
}
const Fragment = ({ children }) => toRawHtml(children);
exports.Fragment = Fragment;
// https://html.spec.whatwg.org/multipage/syntax.html#void-elements
const voidElements = new Set([
    "area",
    "base",
    "br",
    "col",
    "embed",
    "hr",
    "img",
    "input",
    "link",
    "meta",
    "param",
    "source",
    "track",
    "wbr",
]);
function h(type, props, ...children) {
    if (typeof type !== "string") {
        return type({ ...props, children });
    }
    else if (voidElements.has(type)) {
        return new RawHtml(`<${type}${toAttributes(props)}/>`);
    }
    else {
        return new RawHtml(`<${type}${toAttributes(props)}>${toRawHtml(children).html}</${type}>`);
    }
}
exports.h = h;
